set rcsId {$Id: cpustats.tcl,v 1.6 1997/07/20 12:36:31 jfontain Exp $}

# moodss: a Modular Object Oriented Dynamic SpreadSheet
# version 1.0, Copyright (C) 1997 Jean-Luc Fontaine
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# mailto:jfontain@mygale.org


# system dependent cpu statistics data access procedures. only linux is supported at this time (no wonder: it's the greatest
# operating system of them all :)

# corresponding pkgIndex.tcl file content:
# package ifneeded cpustats 1.0 "source [file join $dir cpustats.tcl]"

package provide cpustats 1.0

namespace eval cpustats {
    array set data {
        updates 0
        0,label user 0,type real 0,message {percentage spent in user mode}
        1,label system 1,type real 1,message {percentage spent in system mode}
        2,label nice 2,type real 2,message {percentage spent in nice mode}
        3,label idle 3,type real 3,message {percentage spent in idle mode}
        pollTimes {10 5 20 30 60 120 300}
        columns 4
        sort {0 increasing}
        helpText {
This is a view of processor activity for the system as a whole.
        }
    }

    array set last {user 0 nice 0 system 0 idle 0}                                                    ;# stored values for last poll
    set file [open /proc/stat]                                               ;# keep the statistics file open for better performance

    proc update {} {                                               ;# gather cpu statistics (based on the proc man page information)
        variable file
        variable last
        variable data

        seek $file 0                                                                         ;# always rewind before retrieving data
        scan [gets $file] {cpu %d %d %d %d} user nice system idle                                       ;# in hundredths of a second
        foreach item {user nice system idle} {         ;# calculate statistics during the last poll period except for the first pass
            set current [set $item]
            set $item [expr {$current-$last($item)}]
            set last($item) $current
        }
        set multiplier [expr {100.0/($user+$nice+$system+$idle)}]
        array set data "
            0,0 [format %.1f [expr {$multiplier*$user}]] 0,1 [format %.1f [expr {$multiplier*$system}]]
            0,2 [format %.1f [expr {$multiplier*$nice}]] 0,3 [format %.1f [expr {$multiplier*$idle}]]
        "
        incr data(updates)
    }
}
