# moodss: a Modular Object Oriented Dynamic SpreadSheet
# version 1.0, Copyright (C) 1997-98 Jean-Luc Fontaine
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# mailto:jfontain@mygale.org

set rcsId {$Id: datagraf.tcl,v 1.4 1998/01/15 22:04:54 jfontain Exp $}

class dataGraph {

    set dataGraph::(colors) {red green blue orange cyan black magenta}
    set dataGraph::(colorIndex) 0
    set dataGraph::(vectorIndex) 0

    proc dataGraph {this parentPath args} composite {
        [::new graph $parentPath -title {} -bufferelements 0 -plotborderwidth 1] $args
    } {
        set path $widget::($this,path)
        $path xaxis configure -min -100 -max 0 -tickfont {helvetica 10} -title {}
        $path yaxis configure -title {} -min 0 -tickfont {helvetica 10}
        $path legend configure -borderwidth 1

        set dataGraph::($this,identifiers) {}                                            ;# initialize list of data line identifiers
        # allow dropping of data cell
        set dataGraph::($this,drop) [::new dropSite\
            -path $path -formats DATACELL -command "dataGraph::addDataLine $this \$dragSite::data(DATACELL)"\
        ]

        composite::complete $this
    }

    proc ~dataGraph {this} {
        delete $dataTable::($this,drop)
    }

### the following procedure may not belong here as it may conflict with the object oriented philosophy ###
    proc new {dataCell canvas} {            ;# public procedure for eventually creating a new graph with manager handles on a canvas
        global messenger

        if {![info exists $dataCell]} {
            error "\"$dataCell\" cell does not exist"
        }
        parseCell $dataCell array row column
        if {[catch {checkType [set ${array}($column,type)]} message]} {
            lifoLabel::flash $messenger $message
            bell
            return
        }

        set graph [::new dataGraph $canvas]
        addDataLine $graph $dataCell
        ::new canvasHandles $canvas -path $widget::($graph,path)
        return $graph
    }

    proc options {this} {
        # force size values
        return [list\
            [list -height height Height 200]\
            [list -width width Width 300]\
        ]
    }

    foreach option {-height -width} {
        proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
    }

    proc parseCell {dataCell arrayName rowName columnName} {
        upvar $arrayName array $rowName row $columnName column

        if {![regexp {^(.*)\(([0-9]+),([0-9]+)\)$} $dataCell dummy array row column]||($row<0)||($column<0)} {
            error "\"$dataCell\" is not a valid array cell"
        }
    }

    proc addDataLine {this dataCell} {                                           ;# must be a fully qualified module data array cell
        global messenger

        if {![info exists $dataCell]} {
            error "\"$dataCell\" cell does not exist"
        }
        parseCell $dataCell array row column
        if {[catch {checkType [set ${array}($column,type)]} message]} {
            lifoLabel::flash $messenger $message
            bell
            return
        }

### assume that first column in row uniquely identifies row ###
        set identifier "[set ${array}($row,0)] [set ${array}($column,label)]"
        if {[lsearch -exact $dataGraph::($this,identifiers) $identifier]>=0} return               ;# already graphed, abort addition

        set dataGraph::($this,data,$identifier) $dataCell

        # the following trace is reset (to the same) whenever a new data set is added (the module array should be the same)
        trace variable ${array}(updates) w "dataGraph::update $this"                          ;# refresh as module data gets updated

        set number 100                                                      ;### number of data points should be parameterizable ###
        set vector $dataGraph::(vectorIndex)xData     ;### x axis data vector (should be a namespace variable when blt is fixed) ###
        global $vector       ;### should be a variable but it causes a "called Tcl_FindHashEntry on deleted table" error on exit ###
        blt::vector ${vector}([expr {$number+1}])
        for {set index 0} {$index<=$number} {incr index} {     ;# provide a negative scale since current time is on the x axis right
            set ${vector}($index) [expr {$index-$number}]
        }
        $widget::($this,path) element line $identifier -xdata $vector

        set vector $dataGraph::(vectorIndex)yData
        global $vector
        blt::vector ${vector}([expr {$number+1}])                                                              ;# y axis data vector
        $widget::($this,path) element configure $identifier -ydata $vector

        set dataGraph::($this,vector,$identifier) $vector
        incr dataGraph::(vectorIndex)

        $widget::($this,path) element configure $identifier -color [lindex $dataGraph::(colors) $dataGraph::(colorIndex)]
        if {[incr dataGraph::(colorIndex)]>=[llength $dataGraph::(colors)]} {                                      ;# set next color
            set dataGraph::(colorIndex) 0                                                                   ;# circle through colors
        }

        lappend dataGraph::($this,identifiers) $identifier                                                 ;# register new data line
    }

    proc update {this args} {                                    ;# update display using cells data. ignore eventual trace arguments
        foreach identifier $dataGraph::($this,identifiers) {
            set vector $dataGraph::($this,vector,$identifier)
            global $vector
            $vector delete 0                                                          ;# achieve scrolling by deleting first element
            $vector append [set $dataGraph::($this,data,$identifier)]                                     ;# and appending new value
        }
    }

    proc checkType {type} {                           ;# check type for support and eventually generate an error which may be caught
        if {![regexp {^integer|real$} $type]} {
            error "cannot display data of type $type"
        }
    }
}
